<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use App\Exception\Exception;

class WebService
{

    private $connection;
    private $integrationService;
    private $util;

    public function __construct(
        \Doctrine\DBAL\Connection $connection,
        \App\Service\IntegrationService $integrationService,
        \App\Util\Util $util
    ) {

        $this->integrationService = $integrationService;
        $this->util               = $util;
        $this->connection = $connection;
    }

    public function welcome(Request $request, Response $response)
    {
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'connection'  => $this->connection->ping()
        ]);
    }

    private function getVersionInfo()
    {
        $file = file_get_contents(__DIR__ . "/../../../version.json");
        return json_decode($file, true);
    }

    public function runWebService(Request $request, Response $response)
    {

        try {

            global $CLIENT_NAME;
            $config = $CLIENT_NAME;
            $this->basePath          = $config['logPath'];

            $unfilteredOrders = json_decode($this->util->request('GET', 'orders', null), true);
            $orders = [];

            foreach ($unfilteredOrders['data'] as $order) {
                $fileName = $order['id'] . ".txt";
                $saveLogOrder = $this->util->writeLog('order_pending', $fileName, $order);
                if (!$saveLogOrder)
                    array_push($orders, $order);
            }

            $totalOrders = count($orders);
            if ($totalOrders > 0) {
                $dateLog = date("d/m/Y H:i:s");
                $receivedOrders = "$dateLog - {$totalOrders} Pedido(s) recebido(s) \n";

                foreach ($orders as $order) {
                    $processOrder = $this->integrationService->ProcessOrder($order);

                    if ($processOrder['success']) {
                        $route = "order/{$order['id']}/received";
                        $confirmOrder = json_decode($this->util->request('PUT', $route, null), true);
                        $this->util->writeLog('execution', date("d-m-Y"), date("d:m:Y H:i:s") . " - " . "Pedido {$order['id']} integrado.");
                    }
                }
            }
        } catch (Exception $e) {
            $this->connection->rollback();
            $fileName = date("dmy-H-i-s") . '-' . $order['id'];
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            $this->util->writeLog('order_error', $fileName, $msg);
            return $response->withJson([
                'success' => false,
                'message' => "Erro. {$e->getMessage()}",
                'result'  => []
            ]);
        }

        return $response->withJson([
            'success' => true,
            'message' => isset($msg) ? $msg : '',
            'result'  => []
        ]);
    }

    public function debugOrder(Request $request, Response $response)
    {
        try {
            global $CLIENT_NAME;
            $config = $CLIENT_NAME;
            $req = $request->getParsedBody();

            $this->basePath          = $config['logPath'];

            $order = $req;
            $fileName = $order['id'] . ".txt";
            $this->util->writeLog('order_pending', $fileName, $order);
            $ProcessOrder = $this->integrationService->ProcessOrder($order);

            if ($ProcessOrder['success']) {
                $this->util->writeLog('execution', date("d-m-Y"), date("d:m:Y H:i:s") . " - " . "Pedido {$order['id']} integrados");
                return $response->withJson([
                    'success' => true,
                    'message' => 'Pedido integrado \o/',
                    'result'  => []
                ]);
            }
        } catch (Exception $e) {
            $fileName = date("dmy-H-i-s") . '-' . $order['id'];
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            $this->util->writeLog('order_error', $fileName, $msg);
            return $response->withJson([
                'success' => false,
                'message' => "Erro. {$e->getMessage()}",
                'result'  => []
            ]);
        }
    }

    public function sendProducts(Request $request, Response $response)
    {
        try {

            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Inciando serviço envio de produtos.");

            $produtsPartner = json_decode($this->util->request('GET', 'products', null), true);
            var_dump($produtsPartner);die;
            $productsSend = $this->integrationService->validateSend($produtsPartner['data']);

            if (!empty($productsSend)) {
                $req = json_decode($this->util->request('POST', 'products', $productsSend), true);

                if (!$req['success'])
                    throw new \Exception("Erro no envio de produtos. {$req['message']}");
            }

            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Fim de execução de cadastro de produtos.");
            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Inciando Atualização de produtos.");

            $updateProducts = $this->integrationService->ProductsToUpdating($produtsPartner['data']);
            
            if (!empty($updateProducts)) {
                foreach ($updateProducts as $prod) {
                    $route = 'product/' . $prod['code'];
                    $req = json_decode($this->util->request('PUT', $route, $prod['request']), true);

                    if (!$req['success'])
                        $this->util->writeLog('sendProducts_error', '', date("d:m:Y H:i:s") . " - " . "Erro. {$req['success']}");
                }

                if (!$req['success'])
                    throw new \Exception("Erro na atualização de produtos. {$req['message']}");
            }
            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Fim da atualização de produtos.");

            return $response->withJson([
                'success' => true,
                'message' => '',
                'result'  => []
            ]);

        } catch (Exception $e) {
            $this->util->writeLog('sendProducts_error', '', date("d-m-Y H-i-s") . " - " . $e->getMessage() . "\n");
        }
    }

    public function cards(Request $request, Response $response)
    {
        $this->connection->beginTransaction();
        
        try {

            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Inciando serviço atualização de comanda.");
            $getCards = json_decode($this->util->request('GET', 'cards', null), true);

            if (!empty($getCards['data'])) {
                $ValidateCards = $this->integrationService->ValidateCards($getCards['data']);
            }

            $this->connection->commit();
            $this->util->writeLog('sendProducts_execution', '', date("d:m:Y H:i:s") . " - " . "Fim da Atualização de comanda.");

            return $response->withJson([
                'success' => true,
                'message' => '',
                'result'  => []
            ]);

        } catch (Exception $e) {
            $this->util->writeLog('sendProducts_error', '', date("d-m-Y H-i-s") . " - " . $e->getMessage() . "\n");
        }
    }

    public function CanceledProducts(Request $request, Response $response)
    {
        $this->connection->beginTransaction();

        try {
            
            $this->integrationService->CanceledProducts();

            $this->connection->commit();

            return $response->withJson([
                'success' => true,
                'message' => '',
                'result'  => []
            ]);

        } catch (Exception $e) {
            $this->util->writeLog('canceled_error', '', date("d-m-Y H-i-s") . " - " . $e->getMessage() . "\n");
        }
    }

    public function orderSynchronization(Request $request, Response $response)
    {
        $this->connection->beginTransaction();

        try {            
            $this->integrationService->orderSynchronization();
            $this->integrationService->closeOrder();
            $this->integrationService->transferProducts();
            $this->connection->commit();

            return $response->withJson([
                'success' => true,
                'message' => '',
                'result'  => []
            ]);

        } catch (Exception $e) {
            $this->util->writeLog('canceled_error', '', date("d-m-Y H-i-s") . " - " . $e->getMessage() . "\n");
            return $response->withJson([
                'success' => true,
                'message' => 'Falha na sincronização',
                'result'  => []
            ]);
        }
    }
}
